"""
StashConnect
~~~~~~~~~~~~
An API wrapper for [Stashcat](https://stashcat.com/en).

:copyright: (c) 2024-present BuStudios
:license: MIT, see LICENSE.
"""

__version__ = "0.9.7.6"

from .client import *
